#
#   Functions declaration.
#   Copyright (C) 2010 2011 2014 2023  Alessandro Accattini
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
##############################################################################
#
#
#


#
# Funzioni di conversione
#
dB(a)=20.0*log10(a)
Rad(ang)=2.0*pi*ang/360.0

#
# Creazione di una discontinuita' (polo o zero) in coordinate polari
#
Disc(r,rad)=r*(cos(rad)+{0.0,1.0}*sin(rad))

#
# Misura della distanza tra punti complessi del piano
#
Dist(z,i)=abs(z-i)

#
# Coordinata z (complessa) in funzione della frequenza normalizzata
#
z(fn)=exp({0.0,1.0}*fn*2.0*pi)



#
# Funzioni per il disegno della risposta in frequenza in coordinate polari
#
m_pq(xp,yp,xq,yq)=(yp-yq)/(xp-xq)
q_pq(xp,yp,xq,yq)=yq-(m_pq(xp,yp,xq,yq)*xq)
y_pq(xp,yp,xq,yq,x)=m_pq(xp,yp,xq,yq)*x+q_pq(xp,yp,xq,yq)
lim(min,max,x)=x<=min ? min : x>=max ? max : x
Pol(min,r1,max,a)=y_pq(min,0.25,r1,1.00,lim(min,max,a))
Pol_dB(min,r1,max,a)=Pol(min,r1,max,dB(a))
zPol(x)=z(x/(2.0*pi))



#
# Funzioni per il posizionamento degli zeri di un PID
#
r(Si,Sd)=sqrt(Si*Sd/(Si*Sd + Si + 1.0))
fi(Si,Sd)=acos((1.0+2.0*Sd)*Si/(2.0*sqrt(Si*Sd + Si + 1.0)*sqrt(Si*Sd)))
Lim_r(Sd)=sqrt(Sd/(Sd+1))
Lim_fi(Sd)=acos((0.5+Sd)/sqrt(Sd*(Sd+1)))

#
# Discontinuita' di un PID
#
pid_q1(Si,Sd)=Disc(r(Si,Sd), fi(Si,Sd))
pid_q2(Si,Sd)=Disc(r(Si,Sd),-fi(Si,Sd))
pid_p1=Disc(1.0, 0.0)

#
# Equazione generale del PID, Kp = 1
#
Pid(Si,Sd,z)=  Dist(z,pid_q1(Si,Sd)) * Dist(z,pid_q2(Si,Sd)) / Dist(z,pid_p1)



